@@ -31,8 +31,7 @@ public :
     void    Open_Buffer_Init        (File__Analyze* Sub);
     void    Open_Buffer_Init        (File__Analyze* Sub, int64u File_Size);
     void    Open_Buffer_OutOfBand   (                    const int8u* Buffer, size_t Buffer_Size) {File__Analyze::Buffer=Buffer; File__Analyze::Buffer_Size=Buffer_Size; Element_Offset=0; Element_Size=Buffer_Size; Read_Buffer_OutOfBand(); File__Analyze::Buffer=NULL; File__Analyze::Buffer_Size=0; Element_Offset=0; Element_Size=0;}
-    void    Open_Buffer_OutOfBand   (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size);
-    void    Open_Buffer_OutOfBand   (File__Analyze* Sub) {if (Element_Offset<=Element_Size) Open_Buffer_OutOfBand(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset)); Element_Offset=Element_Size;}
+    void    Open_Buffer_OutOfBand   (File__Analyze* Sub);
     void    Open_Buffer_Continue    (                    const int8u* Buffer, size_t Buffer_Size);
     void    Open_Buffer_Continue    (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size, bool IsNewPacket=true, float64 Ratio=1.0);
     void    Open_Buffer_Continue    (File__Analyze* Sub, size_t Buffer_Size) {if (Element_Offset+Buffer_Size<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, Buffer_Size); Element_Offset+=Buffer_Size;}
@@ -55,7 +54,7 @@ public :
     //***************************************************************************
 
     //In
-    Ztring ParserName;
+    std::string ParserName;
     #if MEDIAINFO_EVENTS
         size_t  StreamIDs_Size;
         int64u  StreamIDs[16];
@@ -74,6 +73,25 @@ public :
     #endif //MEDIAINFO_DEMUX
     Ztring  File_Name_WithoutDemux;
     bool   PTS_DTS_Needed;
+    enum ts_type
+    {
+        TS_NONE=0,
+        TS_PTS=1,
+        TS_DTS=2,
+        TS_ALL=TS_PTS|TS_DTS,
+    };
+    void   TS_Clear(ts_type Type=TS_ALL);
+    void   TS_Set(int64s Ticks, ts_type Type=TS_ALL);
+    void   TS_Set(File__Analyze* Parser, ts_type Type=TS_ALL);
+    void   TS_Add(int64s Ticks, ts_type Type=TS_ALL);
+    void   TS_Ajust(int64s Ticks);
+    int64s Frequency_c; //Frequency of the timestamp of the container (e.g. 90000 for MPEG-PS)
+    int64s Frequency_b; //Frequency of the timestamp of the bitstream (e.g. 48000 for AC-3)
+    #if MEDIAINFO_ADVANCED2
+    static const int64s NoTs=0x8000000000000000LL;
+    int64s PTSb; //In 1/Frequency_b if sub, in 1/(Frequency_c*Frequency_b) if in a container
+    int64s DTSb; //In 1/Frequency_b if sub, in 1/(Frequency_c*Frequency_b) if in a container
+    #endif //MEDIAINFO_ADVANCED2
     struct frame_info
     {
         int64u Buffer_Offset_End;
@@ -81,6 +99,11 @@ public :
         int64u PTS; //In nanoseconds
         int64u DTS; //In nanoseconds
         int64u DUR; //In nanoseconds
+        #if MEDIAINFO_ADVANCED2
+        int64s PTSc; //In 1/Frequency_c
+        int64s DTSc; //In 1/Frequency_c
+        int64u Frame_Count_AfterLastTimeStamp;
+        #endif //MEDIAINFO_ADVANCED2
 
         frame_info()
         {
@@ -89,6 +112,11 @@ public :
             PTS=(int64u)-1;
             DTS=(int64u)-1;
             DUR=(int64u)-1;
+            #if MEDIAINFO_ADVANCED2
+            PTSc=NoTs;
+            DTSc=NoTs;
+            Frame_Count_AfterLastTimeStamp=0;
+            #endif //MEDIAINFO_ADVANCED2
         }
     };
     frame_info FrameInfo;
@@ -118,6 +146,20 @@ public :
         };
         servicedescriptors* ServiceDescriptors;
     #endif
+    #if defined(MEDIAINFO_TELETEXT_YES)
+        struct teletext
+        {
+            std::map<std::string, Ztring>           Infos;
+            stream_t                                StreamKind;
+            size_t                                  StreamPos;
+
+            teletext()
+                : StreamKind(Stream_Max)
+                , StreamPos((size_t)-1)
+            {}
+        };
+        std::map<int16u, teletext>*                 Teletexts; //Key is teletext_magazine_number
+    #endif
 
     //Out
     int64u PTS_Begin;                  //In nanoseconds
@@ -165,7 +207,7 @@ protected :
 
     //Buffer
     virtual void Read_Buffer_Init ()          {}; //Temp, should be in File__Base caller
-    virtual void Read_Buffer_OutOfBand ()     {}; //Temp, should be in File__Base caller
+    virtual void Read_Buffer_OutOfBand ()     {Open_Buffer_Continue(Buffer, Buffer_Size);} //Temp, should be in File__Base caller
     virtual void Read_Buffer_Continue ()      {}; //Temp, should be in File__Base caller
     virtual void Read_Buffer_AfterParsing ()  {}; //Temp, should be in File__Base caller
     #if MEDIAINFO_SEEK
@@ -275,6 +317,7 @@ protected :
     #define Element_Info3(_A,_B,_C)
     #define Element_Info1C(_CONDITION,_A)
     #define Element_Info_From_Milliseconds(_A)
+    #define Element_Parser(_A)
 
     //Elements - End
     inline void Element_End () {Element_End_Common_Flush();}
@@ -289,7 +332,6 @@ protected :
 protected :
     //Element - Common
     void   Element_End_Common_Flush();
-    Ztring Element_End_Common_Flush_Build();
 public :
 
     //***************************************************************************
@@ -297,24 +339,25 @@ public :
     //***************************************************************************
 
     //Param - Main
-    #define Param1(_A)
-    #define Param2(_A,_B)
-    #define Param3(_A,_B,_C)
+    inline void noop() {}
+    #define Param1(_A) noop()
+    #define Param2(_A,_B) noop()
+    #define Param3(_A,_B,_C) noop()
 
     //Param - Info
-    #define Param_Info1(_A)
-    #define Param_Info2(_A,_B)
-    #define Param_Info3(_A,_B,_C)
-    #define Param_Info1C(_CONDITION,_A)
-    #define Param_Info2C(_CONDITION,_A,_B)
-    #define Param_Info3C(_CONDITION,_A,_B,_C)
-    #define Param_Info_From_Milliseconds(A)
+    #define Param_Info1(_A) noop()
+    #define Param_Info2(_A,_B) noop()
+    #define Param_Info3(_A,_B,_C) noop()
+    #define Param_Info1C(_CONDITION,_A) noop()
+    #define Param_Info2C(_CONDITION,_A,_B) noop()
+    #define Param_Info3C(_CONDITION,_A,_B,_C) noop()
+    #define Param_Info_From_Milliseconds(A) noop()
 
     //***************************************************************************
     // Information
     //***************************************************************************
 
-    inline void Info (const Ztring&, size_t =0) {}
+    inline void Info (const std::string&, size_t =0) {}
 
     //***************************************************************************
     // Big Endian (Integer, Float, Fixed-Point)
@@ -1011,7 +1054,13 @@ public :
     #ifdef SIZE_T_IS_LONG
     inline void Fill (stream_t StreamKind, size_t StreamPos, const char* Parameter, size_t         Value, int8u Radix=10, bool Replace=false) {Fill(StreamKind, StreamPos, Parameter, Ztring::ToZtring(Value, Radix).MakeUpperCase(), Replace);}
     #endif //SIZE_T_IS_LONG
-    ZtringListList Fill_Temp;
+    struct fill_temp_item
+    {
+        Ztring Parameter;
+        Ztring Value;
+    };
+    void Fill_SetOptions(stream_t StreamKind, size_t StreamPos, const char* Parameter, const char* Options);
+    vector<fill_temp_item> Fill_Temp[Stream_Max+1]; // +1 because Fill_Temp[Stream_Max] is used when StreamKind is unknown
     void Fill_Flush ();
     static size_t Fill_Parameter(stream_t StreamKind, generic StreamPos);
 
@@ -1084,6 +1133,10 @@ public :
     void CodecID_Fill           (const Ztring &Value, stream_t StreamKind, size_t StreamPos, infocodecid_format_t Format, stream_t StreamKind_CodecID=Stream_Max);
     void PixelAspectRatio_Fill  (const Ztring &Value, stream_t StreamKind, size_t StreamPos, size_t Parameter_Width, size_t Parameter_Height, size_t Parameter_PixelAspectRatio, size_t Parameter_DisplayAspectRatio);
     void DisplayAspectRatio_Fill(const Ztring &Value, stream_t StreamKind, size_t StreamPos, size_t Parameter_Width, size_t Parameter_Height, size_t Parameter_PixelAspectRatio, size_t Parameter_DisplayAspectRatio);
+    #if MEDIAINFO_EVENTS
+    static stream_t Streamkind_Get(int8u* ParserIDs, size_t StreamIDs_Size) {if ((ParserIDs[StreamIDs_Size-1]&0xF0)==0x80) return Stream_Video; if ((ParserIDs[StreamIDs_Size-1]&0xF0)==0xA0) return Stream_Audio; return Stream_Max;}
+    stream_t Streamkind_Get() {return Streamkind_Get(ParserIDs, StreamIDs_Size);}
+    #endif //MEDIAINFO_EVENTS
 
     //***************************************************************************
     // Finalize
@@ -1195,6 +1248,9 @@ protected :
     bool Synchronize_0x000001();
 public:
     void TestContinuousFileNames(size_t CountOfFiles=24, Ztring FileExtension=Ztring(), bool SkipComputeDelay=false);
+    #if MEDIAINFO_FIXITY
+    bool FixFile(int64u FileOffsetForWriting, const int8u* ToWrite, const size_t ToWrite_Size);
+    #endif// MEDIAINFO_FIXITY
 
 private :
 
@@ -1203,9 +1259,9 @@ private :
     //***************************************************************************
 
     //Element
+public: //TO CHANGE
     BitStream_Fast* BS;             //For conversion from bytes to bitstream
     BitStream*      BT;             //For conversion from bytes to bitstream (Little Endian)
-public : //TO CHANGE
     int64u Header_Size;             //Size of the header of the current element
 private :
 
@@ -1328,11 +1384,12 @@ public :
         size_t      AES_Decrypted_Size;
     #endif //MEDIAINFO_AES
 
-    //MD5
-    #if MEDIAINFO_MD5
-        struct MD5Context*  MD5;
-        int64u              Md5_ParseUpTo;
-    #endif //MEDIAINFO_MD5
+    //Hash
+    #if MEDIAINFO_HASH
+        HashWrapper*        Hash;
+        int64u              Hash_Offset;
+        int64u              Hash_ParseUpTo;
+    #endif //MEDIAINFO_HASH
 
     #if MEDIAINFO_SEEK
     private:
